/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_NO_WAIT_INS_TEST_H
#define CUBELIB_POP_NO_WAIT_INS_TEST_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPNoWaitINSTest : public PerformanceTest
{
private:
    cube::Metric* no_wait_ins;
    double        maxValue;

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Instruction Efficiency is the ratio of total number of useful instructions \n"
            "for a reference case (e.g. 1 processor) compared to values when \n"
            "increasing the numbers of processes. \n"
            "A decrease in Instruction Efficiency corresponds to an increase \n"
            "in the total number of instructions required to solve a computational problem. \n";

        std::string not_active_text =
            "Missing Instructions (only computation) is available only, if the measurement has collected \n"
            "either the PAPI counters PAPI_TOT_INS, \n"
            "or the Perf counters instructions and cycles.\n"
            "How to do it see Score-P manual, PERF COUNTERS";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPNoWaitINSTest( cube::CubeProxy* );

    virtual
    ~POPNoWaitINSTest()
    {
    };
// ------ overview tests ---------

    virtual
    inline
    double
    getMaximum() const
    {
        return maxValue;
    }

    virtual
    inline
    bool
    isPercent() const
    {
        return false;
    }

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isNormalized() const // returns true, if range goes from 0 to 1. false -> otherwise
    {
        return false;
    }

    inline
    virtual
    std::string
    units() const
    {
        return "#";
    }
};
};
#endif // POP_ipc_TEST_H
